{$IFDEF VER80}

type
{  TOleEditor }
  TOleEditor = class(TDefaultEditor)
  public
    procedure Edit; override;
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
    procedure Copy; override;
  end;


{ TOleObjTypeProperty
  Property editor the Ole Class (e.g.Word,Excel) and Ole Doc properties   }

  TOleObjTypeProperty = class(TPropertyEditor)
  public
    function  GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
    function  GetValue: string; override;
  end;


{ TOleObjItemProperty
  Property editor the TOleItem - i.e. Paste Special Dlg   }

  TOleObjItemProperty = class(TPropertyEditor)
  public
    function  GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
    function  GetValue: string; override;
  end;

  TLibOle = class (TLibOleHelper)
  public
    function  GetObjectMenuItemCount (Obj : TComponent) : 
                Integer; override;
    function  GetObjectMenuItem (Obj : TComponent; Index : Integer) : 
                String; override;
    procedure ObjectMenuItemAction (Obj : TComponent; Index : Integer);
                override;
    procedure RegisterFormAsDropTarget (Form : TForm);      
                override;
  end;    

{---------------------------------------}
{  TOleEditor }
procedure TOleEditor.Edit;
begin
  TOleContainer(Component).Active := True;
end;


procedure TOleEditor.ExecuteVerb(Index: Integer); 
begin
  TOleContainer(Component).ObjectMenuAction(Index);
end;


function TOleEditor.GetVerb(Index: Integer): string; 
begin
  Result := TOleContainer(Component).GetObjectMenuItem(Index);
end;


function TOleEditor.GetVerbCount: Integer; 
begin
  Result := TOleContainer(Component).GetObjectMenuItemCount;
  if Result > 2 then
    Result := Result - 2;  {remove the convert menu item and spacer}
end;


procedure TOleEditor.Copy; 
begin
  TOleContainer(Component).CopyToClipboard (False);
end;


{ TOleObjTypeProperty }

function TOleObjTypeProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog, paReadOnly]
                      - [paMultiSelect, paSubProperties];
end;


procedure TOleObjTypeProperty.Edit;
var
  Ptr      : Pointer;
  OleCtrl  : TOleContainer;
begin
  OleCtrl := TOleContainer (GetComponent(0));
  if InsertOleObjectDlg (GetParentForm (OleCtrl), 
      THelpContext(hcOleInsertObject), Ptr) = True then
  begin
    if OleCtrl.OleObjAllocated = True then
    begin
      if MessageDlg ( LoadStr(SInitInfo), mtConfirmation, mbOKCancel, 0) <> idOK then
        Exit;
    end;

    OleCtrl.PInitInfo := Ptr;
    ReleaseOleInitInfo (Ptr);
    Modified;
  end;
end;


function TOleObjTypeProperty.GetValue: string;
begin
  if GetStrValue = '' then Result := ''
  else Result := Format ('(%s)', [GetStrValue]);
end;

{-----------------------------------------}

{ TOleObjItemProperty }

function TOleObjItemProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog, paReadOnly] 
                      - [paMultiSelect, paSubProperties];
end;


procedure TOleObjItemProperty.Edit;
var
  Ptr      : Pointer;
  OleCtrl  : TOleContainer;
begin
  OleCtrl := TOleContainer (GetComponent(0));
  if PasteSpecialOleDlg (GetParentForm (OleCtrl), 
      THelpContext (hcOlePasteSpecial), Ptr) = True then
  begin
    if OleCtrl.OleObjAllocated = True then
    begin
      if MessageDlg ( LoadStr(SInitInfo), mtConfirmation, mbOKCancel, 0) <> idOK then 
        Exit;
    end;

    OleCtrl.PInitInfo := Ptr;
    ReleaseOleInitInfo (Ptr);
    Modified;
  end;
end;


function TOleObjItemProperty.GetValue: string;
begin
  Result := GetStrValue;
end;

{-------}

function  TLibOle.GetObjectMenuItemCount (Obj : TComponent) : 
            Integer;
begin
  Result := 0;
  if Obj.InheritsFrom (TOleContainer) then
    Result := TOleContainer (Obj).GetObjectMenuItemCount;
end;


function  TLibOle.GetObjectMenuItem (Obj : TComponent; Index : Integer) : 
            String; 
begin
  Result := EmptyStr;
  if Obj.InheritsFrom (TOleContainer) then
    Result := TOleContainer (Obj).GetObjectMenuItem (Index);
end;


procedure TLibOle.ObjectMenuItemAction (Obj : TComponent; Index : Integer);
begin
  if Obj.InheritsFrom (TOleContainer) then
    TOleContainer (Obj).ObjectMenuAction (Index);
end;


procedure TLibOle.RegisterFormAsDropTarget (Form : TForm);      
begin
  RegisterFormAsOleDropTarget (Form,
    [OleFormat (BoleroObj.EmbedClipFmt, '%s', '%s', TRUE),
     OleFormat (BoleroObj.LinkClipFmt,  '%s', '%s', TRUE)]);
end;

{$ELSE}

{ TOleContainerEditor }

type
  TOleContainerEditor = class(TDefaultEditor)
  private
    FEditVerbs: TStringList;
  public
    constructor Create(AComponent: TComponent; ADesigner: TFormDesigner); override;
    destructor Destroy; override;
    procedure Edit; override;
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

constructor TOleContainerEditor.Create(AComponent: TComponent;
  ADesigner: TFormDesigner);
begin
  inherited Create(AComponent, ADesigner);
  FEditVerbs := TStringList.Create;
end;

destructor TOleContainerEditor.Destroy;
begin
  FEditVerbs.Free;
  inherited Destroy;
end;

procedure TOleContainerEditor.Edit;
begin
  with TOleContainer(Component) do
    if State <> osEmpty then DoVerb(ovPrimary) else InsertObjectDialog;
end;

procedure TOleContainerEditor.ExecuteVerb(Index: Integer);
var
  Action: Integer;
begin
  Action := Integer(FEditVerbs.Objects[Index]);
  with TOleContainer(Component) do
    if Action >= 0 then
      DoVerb(Action)
    else
      case Action of
        -1: if InsertObjectDialog and NewInserted then DoVerb(ovOpen);
        -2: PasteSpecialDialog;
        -3: Copy;
        -4: DestroyObject;
        -5: ObjectPropertiesDialog;
      end;
end;

function TOleContainerEditor.GetVerb(Index: Integer): string;
begin
  Result := FEditVerbs[Index];
end;

{$IFDEF VER100}
const
  SInsertObjectVerb = 'Insert Object';
  SPasteSpecialVerb = 'Paste Special';
  SCopyObjectVerb = 'Copy Object';
  SDeleteObjectVerb = 'Delete Object';
  SObjectPropertiesVerb = 'Object Properties';
{$ENDIF}

function TOleContainerEditor.GetVerbCount: Integer;
var
  I, P: Integer;
  S: string;
begin
  with TOleContainer(Component) do
  begin
    FEditVerbs.Clear;
    if State <> osEmpty then
    begin
      UpdateVerbs;
      if ObjectVerbs.Count > 0 then
      begin
        for I := 0 to ObjectVerbs.Count - 1 do
        begin
          S := ObjectVerbs[I];
          for P := Length(S) downto 1 do
            if S[P] = '&' then Delete(S, P, 1);
          FEditVerbs.AddObject(S, TObject(I));
        end;
        FEditVerbs.Add('-');
      end;
    end;
    {$IFDEF VER100}
    FEditVerbs.AddObject(SInsertObjectVerb, TObject(-1));
    if CanPaste then
      FEditVerbs.AddObject(SPasteSpecialVerb, TObject(-2));
    if State <> osEmpty then
    begin
      FEditVerbs.AddObject(SCopyObjectVerb, TObject(-3));
      FEditVerbs.AddObject(SDeleteObjectVerb, TObject(-4));
      FEditVerbs.AddObject(SObjectPropertiesVerb, TObject(-5));
    end;
    {$ELSE}
    FEditVerbs.AddObject(LoadStr(SInsertObjectVerb), TObject(-1));
    if CanPaste then
      FEditVerbs.AddObject(LoadStr(SPasteSpecialVerb), TObject(-2));
    if State <> osEmpty then
    begin
      FEditVerbs.AddObject(LoadStr(SCopyObjectVerb), TObject(-3));
      FEditVerbs.AddObject(LoadStr(SDeleteObjectVerb), TObject(-4));
      FEditVerbs.AddObject(LoadStr(SObjectPropertiesVerb), TObject(-5));
    end;
    {$ENDIF}
  end;
  Result := FEditVerbs.Count;
end;

{$ENDIF}

