{$IFDEF VER80}
type
  TTabPageNameProperty = class(TPropertyEditor)
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    function GetValue: string; override;
    procedure SetValue(const AValue: string); override;
  end;

  TTabPageListProperty = class(TPropertyEditor)
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure Edit; override;
  end;

  TTabbedNotebookEditor = class(TDefaultEditor)
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

{ TTabPageNameProperty }
function TTabPageNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

procedure TTabPageNameProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  Notebook: TTabbedNotebook;
  Page: TTabPage;
begin
  Notebook := GetComponent(0) as TTabbedNotebook;
  for I := 0 to Notebook.Pages.Count - 1 do
  begin
    Page := Notebook.Pages.Objects[I] as TTabPage;
    Proc(Page.Caption);
  end;
end;

function TTabPageNameProperty.GetValue: string;
begin
  Result := GetStrValue;
end;

procedure TTabPageNameProperty.SetValue(const AValue: string);
begin
  SetStrValue(AValue);
end;

{ TTabPageListProperty }
function TTabPageListProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

function TTabPageListProperty.GetValue: string;
begin
  FmtStr(Result, '(%s)', [GetPropType^.Name]);
end;

procedure TTabPageListProperty.Edit;
var
  D: TNotebookEditDlg;
begin
  D := TNotebookEditDlg.Create(Application);
  try
    D.SetPageList(TStringList(GetOrdValue));
    D.ShowModal;
    if D.Modified then Designer.Modified;
  finally
    D.Free;
  end;
end;

{ TNotebookEditor }

const
  Verbs: array[0..1] of Word = (SNextPage, SPrevPage);

procedure TTabbedNotebookEditor.ExecuteVerb(Index: Integer);
var
  NewPage: Integer;
begin
  with TTabbedNotebook(Component) do
  begin
    NewPage := PageIndex;
    if Index = 0 then
      Inc(NewPage) else
      Dec(NewPage);
    if NewPage < 0 then NewPage := Pages.Count
    else if NewPage >= Pages.Count then NewPage := 0;
    if NewPage <> PageIndex then
    begin
      PageIndex := NewPage;
      Designer.Modified;
    end;
  end;
end;

function TTabbedNotebookEditor.GetVerb(Index: Integer): string;
begin
  Result := LoadStr(Verbs[Index]);
end;

function TTabbedNotebookEditor.GetVerbCount: Integer;
begin
  Result := High(Verbs) + 1;
end;

{$ELSE}

{ TActivePageProperty }

type
  TActivePageProperty = class(TComponentProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

function TActivePageProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

procedure TActivePageProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  Component: TComponent;
begin
  for I := 0 to Designer.Form.ComponentCount - 1 do
  begin
    Component := Designer.Form.Components[I];
    if (Component.Name <> '') and (Component is TTabSheet) and
      (TTabSheet(Component).PageControl = GetComponent(0)) then
      Proc(Component.Name);
  end;
end;

{ TPageControlEditor }

type
  TPageControlEditor = class(TDefaultEditor)
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

const
  {$IFDEF VER100}
  SNewPage = 0;
  SNextPage = 1;
  SPrevPage = 2;
  {$ENDIF}
  PageControlVerbs: array[0..2] of Word = (SNewPage, SNextPage, SPrevPage);

procedure TPageControlEditor.ExecuteVerb(Index: Integer);
var
  PageControl: TPageControl;
  Page: TTabSheet;
  Designer: TFormDesigner;
begin
  if Component is TTabSheet then
    PageControl := TTabSheet(Component).PageControl else
    PageControl := TPageControl(Component);
  if PageControl <> nil then
  begin
    Designer := Self.Designer;
    if Index = 0 then
    begin
      Page := TTabSheet.Create(Designer.Form);
      try
        Page.Name := Designer.UniqueName(TTabSheet.ClassName);
        Page.Parent := PageControl;
        Page.PageControl := PageControl;
      except
        Page.Free;
        raise;
      end;
      PageControl.ActivePage := Page;
      Designer.SelectComponent(Page);
      Designer.Modified;
    end else
    begin
      Page := PageControl.FindNextPage(PageControl.ActivePage,
        Index = 1, False);
      if (Page <> nil) and (Page <> PageControl.ActivePage) then
      begin
        PageControl.ActivePage := Page;
        if Component is TTabSheet then Designer.SelectComponent(Page);
        Designer.Modified;
      end;
    end;
  end;
end;

function TPageControlEditor.GetVerb(Index: Integer): string;
begin
  Result := LoadStr(PageControlVerbs[Index]);
end;

function TPageControlEditor.GetVerbCount: Integer;
begin
  Result := High(PageControlVerbs) + 1;
end;

{ TStatusPanelsProperty }

type
  TStatusPanelsProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure TStatusPanelsProperty.Edit;
begin
  {$IFDEF VER90}
  if EditStatusPanels(TStatusPanels(GetOrdValue)) then Modified;
  {$ENDIF}
end;

function TStatusPanelsProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

{ THeaderSectionsProperty }

type
  THeaderSectionsProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure THeaderSectionsProperty.Edit;
begin
  {$IFDEF VER90}
  if EditHeaderSections(THeaderSections(GetOrdValue)) then Modified;
  {$ENDIF}
end;

function THeaderSectionsProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

{ TListColumnsProperty }

type
  TListColumnsProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure TListColumnsProperty.Edit;
begin
  {$IFDEF VER90}
  if EditListViewColumns(TListColumns(GetOrdValue)) then Modified;
  {$ENDIF}
end;

function TListColumnsProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

{ TListItemsProperty }

type
  TListItemsProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure TListItemsProperty.Edit;
begin
  {$IFDEF VER90}
  if EditListViewItems(TListItems(GetOrdValue)) then Modified;
  {$ENDIF}
end;

function TListItemsProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

{ TTreeNodesProperty }

type
  TTreeNodesProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure TTreeNodesProperty.Edit;
begin
  {$IFDEF VER90}
  if EditTreeViewItems(TTreeNodes(GetOrdValue)) then Modified;
  {$ENDIF}
end;

function TTreeNodesProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

type
  TImageListEditor = class(TComponentEditor)
  private
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

procedure TImageListEditor.ExecuteVerb(Index: Integer);
begin
  {$IFDEF VER90}
  case Index of
    0:
      if EditImageList(Component as TImageList) and (Designer <> nil) then
        Designer.Modified;
  end;
  {$ENDIF}
end;

function TImageListEditor.GetVerb(Index: Integer): string;
begin
  {$IFDEF VER90}
  if Index = 0 then Result := LoadStr(SImageEditor);
  {$ENDIF}
end;

function TImageListEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;
{$ENDIF}
